package sun.management;

import java.lang.reflect.InvocationTargetException;
import java.util.Collections;
import java.lang.management.PlatformManagedObject;
import java.util.List;
import jdk.internal.cmm.SystemResourcePressureImpl;
import jdk.management.cmm.SystemResourcePressureMXBean;

public final class ExtendedPlatformComponent
{
    private static SystemResourcePressureMXBean cmmBeanImpl;
    
    private ExtendedPlatformComponent() {
    }
    
    private static synchronized SystemResourcePressureMXBean getCMMBean() {
        if (ExtendedPlatformComponent.cmmBeanImpl == null) {
            ExtendedPlatformComponent.cmmBeanImpl = new SystemResourcePressureImpl();
        }
        return ExtendedPlatformComponent.cmmBeanImpl;
    }
    
    public static List<? extends PlatformManagedObject> getMXBeans() {
        if (shouldRegisterCMMBean()) {
            return Collections.singletonList(getCMMBean());
        }
        return Collections.emptyList();
    }
    
    public static <T extends PlatformManagedObject> T getMXBean(Class<T> clazz) {
        if (clazz != null && "jdk.management.cmm.SystemResourcePressureMXBean".equals(clazz.getName())) {
            if (isUnlockCommercialFeaturesEnabled()) {
                return (T)clazz.cast(getCMMBean());
            }
            throw new IllegalArgumentException("Cooperative Memory Management is a commercial feature which must be unlocked before being used.  To learn more about commercial features and how to unlock them visit http://www.oracle.com/technetwork/java/javaseproducts/");
        }
        else {
            return null;
        }
    }
    
    private static boolean shouldRegisterCMMBean() {
        if (!isUnlockCommercialFeaturesEnabled()) {
            return false;
        }
        boolean booleanValue = false;
        try {
            ClassLoader systemClassLoader = ClassLoader.getSystemClassLoader();
            if (systemClassLoader == null) {
                return false;
            }
            Class<Object> forName = Class.forName("com.oracle.exalogic.ExaManager", false, systemClassLoader.getParent());
            Object invoke = forName.getMethod("instance", new Class[0]).invoke(null, new Object[0]);
            if (invoke != null) {
                booleanValue = (boolean)forName.getMethod("isExalogicSystem", new Class[0]).invoke(invoke, new Object[0]);
            }
            return booleanValue;
        }
        catch (ClassNotFoundException | NoSuchMethodException | IllegalAccessException | IllegalArgumentException | InvocationTargetException ex) {
            return false;
        }
    }
    
    private static boolean isUnlockCommercialFeaturesEnabled() {
        Flag flag = Flag.getFlag("UnlockCommercialFeatures");
        return flag != null && "true".equals(flag.getVMOption().getValue());
    }
    
    static {
        ExtendedPlatformComponent.cmmBeanImpl = null;
    }
}
